@@ -1,13 +1,12 @@
 use crate::conn::{CommonState, ConnectionRandoms, State};
-#[cfg(feature = "tls12")]
-use crate::enums::CipherSuite;
-use crate::enums::{ProtocolVersion, SignatureScheme};
 use crate::error::Error;
 use crate::hash_hs::{HandshakeHash, HandshakeHashBuffer};
 #[cfg(feature = "logging")]
 use crate::log::{debug, trace};
-use crate::msgs::enums::HandshakeType;
+#[cfg(feature = "tls12")]
+use crate::msgs::enums::CipherSuite;
 use crate::msgs::enums::{AlertDescription, Compression, ExtensionType};
+use crate::msgs::enums::{HandshakeType, ProtocolVersion, SignatureScheme};
 #[cfg(feature = "tls12")]
 use crate::msgs::handshake::SessionID;
 use crate::msgs::handshake::{ClientHelloPayload, Random, ServerExtension};
@@ -271,7 +270,7 @@ impl ExpectClientHello {
     /// Continues handling of a `ClientHello` message once config and certificate are available.
     pub(super) fn with_certified_key(
         self,
-        mut sig_schemes: Vec<SignatureScheme>,
+        sig_schemes: Vec<SignatureScheme>,
         client_hello: &ClientHelloPayload,
         m: &Message,
         cx: &mut ServerContext<'_>,
@@ -316,26 +315,6 @@ impl ExpectClientHello {
 
         cx.common.negotiated_version = Some(version);
 
-        // We communicate to the upper layer what kind of key they should choose
-        // via the sigschemes value.  Clients tend to treat this extension
-        // orthogonally to offered ciphersuites (even though, in TLS1.2 it is not).
-        // So: reduce the offered sigschemes to those compatible with the
-        // intersection of ciphersuites.
-        let client_suites = self
-            .config
-            .cipher_suites
-            .iter()
-            .copied()
-            .filter(|scs| {
-                client_hello
-                    .cipher_suites
-                    .contains(&scs.suite())
-            })
-            .collect::<Vec<_>>();
-
-        sig_schemes
-            .retain(|scheme| suites::compatible_sigscheme_for_suites(*scheme, &client_suites));
-
         // Choose a certificate.
         let certkey = {
             let client_hello = ClientHello::new(
@@ -432,8 +411,13 @@ impl ExpectClientHello {
 
 impl State<ServerConnectionData> for ExpectClientHello {
     fn handle(self: Box<Self>, cx: &mut ServerContext<'_>, m: Message) -> NextStateOrError {
-        let (client_hello, sig_schemes) =
-            process_client_hello(&m, self.done_retry, cx.common, cx.data)?;
+        let (client_hello, sig_schemes) = process_client_hello(
+            &m,
+            self.done_retry,
+            &self.config.cipher_suites,
+            cx.common,
+            cx.data,
+        )?;
         self.with_certified_key(sig_schemes, client_hello, &m, cx)
     }
 }
@@ -448,6 +432,7 @@ impl State<ServerConnectionData> for ExpectClientHello {
 pub(super) fn process_client_hello<'a>(
     m: &'a Message,
     done_retry: bool,
+    supported_cipher_suites: &[SupportedCipherSuite],
     common: &mut CommonState,
     data: &mut ServerConnectionData,
 ) -> Result<(&'a ClientHelloPayload, Vec<SignatureScheme>), Error> {
@@ -507,10 +492,26 @@ pub(super) fn process_client_hello<'a>(
         ));
     }
 
-    let sig_schemes = client_hello
+    // We communicate to the upper layer what kind of key they should choose
+    // via the sigschemes value.  Clients tend to treat this extension
+    // orthogonally to offered ciphersuites (even though, in TLS1.2 it is not).
+    // So: reduce the offered sigschemes to those compatible with the
+    // intersection of ciphersuites.
+    let client_suites = supported_cipher_suites
+        .iter()
+        .copied()
+        .filter(|scs| {
+            client_hello
+                .cipher_suites
+                .contains(&scs.suite())
+        })
+        .collect::<Vec<_>>();
+
+    let mut sig_schemes = client_hello
         .get_sigalgs_extension()
         .ok_or_else(|| incompatible(common, "client didn't describe signature schemes"))?
         .clone();
+    sig_schemes.retain(|scheme| suites::compatible_sigscheme_for_suites(*scheme, &client_suites));
 
     Ok((client_hello, sig_schemes))
 }
